
import java.io.*;
import java.security.*;
import org.bouncycastle.jce.provider.*;

/** Genera un par de claves utilizando RSA
 *  Uso: java GeneraClavesRSA fich_clave_publica fich_clave_privada
 */
public class GeneraClavesRSA {

   public static void main(String[] args) {
      try {
         // Indicamos que deseamos utilizar un proveedor de algoritmos
         // de seguridad (si no se hace esto se lanza una excepcion)
         Security.addProvider(new BouncyCastleProvider());
         
         // Devuelve un generador de claves que utiliza el algoritmo pasado
         // como primer argumento y cuyo proveedor es el segundo argumento
         // (es el proveedor que hemos instalado en la sentencia anterior)
         KeyPairGenerator pairgen = KeyPairGenerator.getInstance("RSA","BC");
         
         // Semilla aleatoria
         SecureRandom random = new SecureRandom();
                  
         pairgen.initialize(1024,random);
         
         // Generacion del par de claves
         
         KeyPair parClaves = pairgen.generateKeyPair();
         
         System.out.println("Tamao de la clave publica: " + parClaves.getPublic().getEncoded().length);
         System.out.println("Tamao de la clave privada: " + parClaves.getPrivate().getEncoded().length);
         
         // Escritura de la clave publica
         
         ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(args[0]));
         
         out.writeObject(parClaves.getPublic());
         
         out.close();
         
         // Escritura de la clave privada
         out = new ObjectOutputStream(new FileOutputStream(args[1]));
         
         out.writeObject(parClaves.getPrivate());
         
         out.close();
               
         
      } catch (NoSuchAlgorithmException e) {
         e.printStackTrace();
      } catch (NoSuchProviderException e) {
         e.printStackTrace();
      } catch (FileNotFoundException e) {
         e.printStackTrace();
      } catch (IOException e) {
         e.printStackTrace();
      }
      
   }
}