import java.io.*;
import java.security.*;
import javax.crypto.*;
import org.bouncycastle.jce.provider.*;


/** Clase que se puede utilizar para desencriptar un mensaje.
 *  Utiliza la clave privada para desencriptar la clave simetrica.
 *  El mensaje es desencriptado con la clave simetrica.
 * @author Juan
 *
 */
public class DesencriptaMensaje {

   public static void main(String[] args) {
      
      // Con esto indicamos que deseamos utilizar un proveedor de clases
      // de seguridad (para poder pasar el parametro "BC") 
		Security.addProvider(new BouncyCastleProvider());
      
		if (args.length==0){
			System.out.println("USO: java Desencripta fichero_clave_privada fichero_clave_encriptada fichero_datos_encriptado fichero_datos");
			System.exit(0);			
		}
      try {
         // El proceso es el siguiente:
         // 1) Se leen el numero de bytes que ocupa la clave secreta y se leen
         //    esos bytes del fichero obteniendo asi la clave secreta codificada
         //    con la clave publica 
         // 2) Se desencripta la clave simetrica utilizando la clave privada
         // 3) Se leen los bytes del fichero y se desencriptan utilizando la clave
         //    simetrica

         // Lectura de la clave simetrica codificada:

         DataInputStream in = new DataInputStream(new FileInputStream(args[1]));
         int longitud = in.readInt();
         byte[] claveSimetricaCodificada = new byte[longitud];
         in.read(claveSimetricaCodificada, 0, longitud);
         in.close();
         
         System.out.println("Longitud de la clave simetrica codificada: " + longitud);

         // Desencriptar la clave simetrica con la clave privada

         // a) Leer la clave privada

         ObjectInputStream ficheroClave =
            new ObjectInputStream(new FileInputStream(args[0]));
         Key clavePrivada = (Key) ficheroClave.readObject();
         ficheroClave.close();

         System.out.println("Longitud de la clave privada: " + clavePrivada.getEncoded().length);


         // b) Desencriptar la clave simetrica

         Cipher cifrador = Cipher.getInstance("RSA","BC");
         cifrador.init(Cipher.UNWRAP_MODE, clavePrivada);
         Key claveSimetrica = cifrador.unwrap(claveSimetricaCodificada, "RSA", Cipher.SECRET_KEY);

         System.out.println("Longitud de la clave simetrica decodificada: " + claveSimetrica.getEncoded().length);


         // Lectura del fichero desencriptando con la clave simetrica

         
         Cipher simetrico = Cipher.getInstance("AES","BC");
         simetrico.init(Cipher.DECRYPT_MODE, claveSimetrica);

         InputStream fichCod = new BufferedInputStream(new FileInputStream(args[2]));
         OutputStream fich = new BufferedOutputStream(new FileOutputStream(args[3]));

         Cifra.cifra(fichCod,fich,simetrico);

         fich.close();
         fichCod.close();        

      } catch (IOException e) {
         e.printStackTrace();
      } catch (ClassNotFoundException e) {
         e.printStackTrace();
      } catch (GeneralSecurityException e) {
         e.printStackTrace();
      }
   	
   }
}
